#include "ISocket.h"
#include "SocketClient.h"
#include "TamponSecurise.h"
#include "Autonome.h"
#include <iostream>
using std::cerr;
using std::cin;
using std::endl;
using std::cout;
#include <utility>
using std::pair;
#include <string>
using std::string;
using std::getline;

template <class T, class C>
   class EmetteurAsynchrone
      : public Autonome
   {
      T &sortie_;
      volatile TamponSecurise <C> &a_envoyer_;
      EmetteurAsynchrone(T &sortie, volatile TamponSecurise<C> &t)
         : Autonome(100), sortie_(sortie), a_envoyer_(t)
      {
      }
      C pret_;
   public:
      typedef typename
         C::size_type size_type;
      static EmetteurAsynchrone *creer(T &sortie, volatile TamponSecurise<C> &t)
      {
         EmetteurAsynchrone *p = new EmetteurAsynchrone(sortie, t);
         p->demarrer();
         return p;
      }
      void agir()
      {
         if (!a_envoyer_.empty())
         {
            C temp = a_envoyer_.extraire();
            pret_.append(temp.begin(), temp.end());
         }
         if (!pret_.empty())
         {
            try
            {
               size_type n = sortie_.envoyer<char>(pret_.begin(), pret_.end());
               if (n)
               {
                  typedef typename
                     C::iterator iterator;
                  iterator itt = pret_.begin();
                  std::advance(itt, n);
                  pret_.erase(pret_.begin(), itt);
               }
            }
            catch (pair<SocketsFlux::ISocket::FermetureHomologue,
                        SocketsFlux::ISocket::erreur_type> &p)
            {
               if (p.first.brutale())
                  cerr << "Fermeture brutale de l'homologue" << endl;
               else
                  cerr << "Fermeture propre de l'homologue" << endl;
               demande_arret();
            }
         }
      }
   };

#include <memory>
using std::auto_ptr;

int main()
{
   const SocketsFlux::Adresse ADRESSE_SERVEUR ("127.0.0.1");
   const SocketsFlux::Port PORT_SERVEUR (15234);
   SocketsFlux::SocketClient client(ADRESSE_SERVEUR, PORT_SERVEUR);
   TamponSecurise<string> tampon;
   auto_ptr<EmetteurAsynchrone <SocketsFlux::SocketClient, string> > emetteur
      (EmetteurAsynchrone<SocketsFlux::SocketClient, string>::creer(client, tampon));
   const string FIN = ".";
   cout << "Entrez le texte  transmettre (\"" << FIN << "\" pour complter)"
        << endl;
   string ligne;
   while (getline(cin, ligne) && ligne != FIN)
      tampon.ajouter(ligne + "\n");
}
